"""Interactive machine learning supervision."""
from .class_labeller import ClassLabeller
from .multiclass_labeller import MultiClassLabeller

__all__ = ["MultiClassLabeller", "ClassLabeller"]
__version__ = "0.5.0rc1"
