#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['superintendent',
 'superintendent.acquisition_functions',
 'superintendent.controls',
 'superintendent.display',
 'superintendent.distributed',
 'superintendent.queueing']

package_data = \
{'': ['*']}

install_requires = \
['ipywidgets>=7.4',
 'numpy>=1.15',
 'pandas>=0.24.0',
 'scikit-learn>=0.20.3',
 'scipy>=1.2',
 'schedule>=0.6.0',
 'sqlalchemy>=1.3',
 'Pillow',
 'cachetools>=3.1',
 'psycopg2-binary>=2.8',
 'flask>=1.0',
 'ipyevents>=0.6.0']

extras_require = \
{'dev': ['jupyterlab>=1.1', 'rope', 'docargs', 'black>=19.3b0', 'flake8>=3.7'],
 'doc': ['sphinx>=2.0',
         'requests>=2.21',
         'html5lib>=1.0',
         'bs4>=0.0.1',
         'wordcloud>=1.5',
         'm2r>=0.2.1',
         'sphinx_rtd_theme>=0.4.3',
         'jupyter-sphinx>=0.1.4',
         'nbsphinx>=0.4.2'],
 'examples': ['requests>=2.21',
              'html5lib>=1.0',
              'bs4>=0.0.1',
              'wordcloud>=1.5'],
 'test': ['isort>=4.3.21',
          'pytest>=4.1',
          'pytest-cov>=2.6',
          'hypothesis>=4.17',
          'pytest-helpers-namespace>=2019.1',
          'pytest-mock>=1.10',
          'nbconvert>=5.4',
          'coveralls>=1.7']}

setup(name='superintendent',
      version='0.5.0rc1',
      description='Interactive machine learning supervision.',
      author='Jan Freyberg',
      author_email='jan.freyberg@gmail.com',
      url='https://github.com/janfreyberg/superintendent',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )
